-- The layer4 of Convolutional Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2022/01/02

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity conv_layer4 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  OutChSel : in std_logic;
  CO300 : in std_logic_vector(8 downto 0);
  CO9 : in std_logic_vector(3 downto 0);
  CO128 : in std_logic_vector(6 downto 0);
  CO128_dly2 : in std_logic_vector(6 downto 0);
  X_RD_DATA : in std_logic_vector(639 downto 0); 
  ReluAll : out std_logic_vector(639 downto 0); 
  ReluAllWrN : out std_logic; 
  ReluSum : out std_logic_vector(11 downto 0) ); 
end conv_layer4;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of conv_layer4 is

component dotpro_W4
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  PixRomData : in std_logic_vector(4 downto 0); 
  MatRomData : in std_logic_vector(2 downto 0);
  CO9is1 : in std_logic;
  CO128is0 : in std_logic;
  MATMULOUT : Out std_logic_vector(13 downto 0) );  
end component;

component ROM_W4
Port (
          CLK : In std_logic;
          ADDR : In std_logic_vector(10 downto 0);
          DATA : Out std_logic_vector(383 downto 0) );
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal CO9_clr : std_logic;
signal W3BUSY, W3BUSY_dly1, W3BUSY_dly2, W3BUSY_dly3, W3BUSY_dly4 : std_logic;

signal mac_00, mac_01, mac_02, mac_03, mac_04, mac_05, mac_06, mac_07, mac_08, mac_09 : std_logic_vector(13 downto 0);
signal mac_10, mac_11, mac_12, mac_13, mac_14, mac_15, mac_16, mac_17, mac_18, mac_19 : std_logic_vector(13 downto 0);
signal mac_20, mac_21, mac_22, mac_23, mac_24, mac_25, mac_26, mac_27, mac_28, mac_29 : std_logic_vector(13 downto 0);
signal mac_30, mac_31, mac_32, mac_33, mac_34, mac_35, mac_36, mac_37, mac_38, mac_39 : std_logic_vector(13 downto 0);
signal mac_40, mac_41, mac_42, mac_43, mac_44, mac_45, mac_46, mac_47, mac_48, mac_49 : std_logic_vector(13 downto 0);
signal mac_50, mac_51, mac_52, mac_53, mac_54, mac_55, mac_56, mac_57, mac_58, mac_59 : std_logic_vector(13 downto 0);
signal mac_60, mac_61, mac_62, mac_63 : std_logic_vector(13 downto 0);

signal truncate_00, truncate_01, truncate_02, truncate_03, truncate_04, truncate_05, truncate_06, truncate_07, truncate_08, truncate_09 : std_logic_vector(6 downto 0);
signal truncate_10, truncate_11, truncate_12, truncate_13, truncate_14, truncate_15, truncate_16, truncate_17, truncate_18, truncate_19 : std_logic_vector(6 downto 0);
signal truncate_20, truncate_21, truncate_22, truncate_23, truncate_24, truncate_25, truncate_26, truncate_27, truncate_28, truncate_29 : std_logic_vector(6 downto 0);
signal truncate_30, truncate_31, truncate_32, truncate_33, truncate_34, truncate_35, truncate_36, truncate_37, truncate_38, truncate_39 : std_logic_vector(6 downto 0);
signal truncate_40, truncate_41, truncate_42, truncate_43, truncate_44, truncate_45, truncate_46, truncate_47, truncate_48, truncate_49 : std_logic_vector(6 downto 0);
signal truncate_50, truncate_51, truncate_52, truncate_53, truncate_54, truncate_55, truncate_56, truncate_57, truncate_58, truncate_59 : std_logic_vector(6 downto 0);
signal truncate_60, truncate_61, truncate_62, truncate_63 : std_logic_vector(6 downto 0);

signal Relu_00, Relu_01, Relu_02, Relu_03, Relu_04, Relu_05, Relu_06, Relu_07, Relu_08, Relu_09 : std_logic_vector(4 downto 0);
signal Relu_10, Relu_11, Relu_12, Relu_13, Relu_14, Relu_15, Relu_16, Relu_17, Relu_18, Relu_19 : std_logic_vector(4 downto 0);
signal Relu_20, Relu_21, Relu_22, Relu_23, Relu_24, Relu_25, Relu_26, Relu_27, Relu_28, Relu_29 : std_logic_vector(4 downto 0);
signal Relu_30, Relu_31, Relu_32, Relu_33, Relu_34, Relu_35, Relu_36, Relu_37, Relu_38, Relu_39 : std_logic_vector(4 downto 0);
signal Relu_40, Relu_41, Relu_42, Relu_43, Relu_44, Relu_45, Relu_46, Relu_47, Relu_48, Relu_49 : std_logic_vector(4 downto 0);
signal Relu_50, Relu_51, Relu_52, Relu_53, Relu_54, Relu_55, Relu_56, Relu_57, Relu_58, Relu_59 : std_logic_vector(4 downto 0);
signal Relu_60, Relu_61, Relu_62, Relu_63 : std_logic_vector(4 downto 0);

signal outCh_000, outCh_001, outCh_002, outCh_003, outCh_004, outCh_005, outCh_006, outCh_007, outCh_008, outCh_009 : std_logic_vector(4 downto 0);

signal PixRomData : std_logic_vector(4 downto 0);
signal W4Addr, W4Addr_prev : std_logic_vector(10 downto 0);
signal W4Addr_L : std_logic_vector(3 downto 0);
signal W4Addr_H : std_logic_vector(10 downto 0);
signal W4Sel : std_logic;
signal OutChSel_dly1, OutChSel_dly2 : std_logic;
signal reg_00_0, reg_00_1 : std_logic_vector(4 downto 0);
signal reg_01_0, reg_01_1 : std_logic_vector(4 downto 0);
signal reg_02_0, reg_02_1 : std_logic_vector(4 downto 0);
signal reg_03_0, reg_03_1 : std_logic_vector(4 downto 0);
signal reg_04_0, reg_04_1 : std_logic_vector(4 downto 0);
signal reg_05_0, reg_05_1 : std_logic_vector(4 downto 0);
signal reg_06_0, reg_06_1 : std_logic_vector(4 downto 0);
signal reg_07_0, reg_07_1 : std_logic_vector(4 downto 0);
signal reg_08_0, reg_08_1 : std_logic_vector(4 downto 0);
signal reg_09_0, reg_09_1 : std_logic_vector(4 downto 0);
signal reg_10_0, reg_10_1 : std_logic_vector(4 downto 0);
signal reg_11_0, reg_11_1 : std_logic_vector(4 downto 0);
signal reg_12_0, reg_12_1 : std_logic_vector(4 downto 0);
signal reg_13_0, reg_13_1 : std_logic_vector(4 downto 0);
signal reg_14_0, reg_14_1 : std_logic_vector(4 downto 0);
signal reg_15_0, reg_15_1 : std_logic_vector(4 downto 0);
signal reg_16_0, reg_16_1 : std_logic_vector(4 downto 0);
signal reg_17_0, reg_17_1 : std_logic_vector(4 downto 0);
signal reg_18_0, reg_18_1 : std_logic_vector(4 downto 0);
signal reg_19_0, reg_19_1 : std_logic_vector(4 downto 0);
signal reg_20_0, reg_20_1 : std_logic_vector(4 downto 0);
signal reg_21_0, reg_21_1 : std_logic_vector(4 downto 0);
signal reg_22_0, reg_22_1 : std_logic_vector(4 downto 0);
signal reg_23_0, reg_23_1 : std_logic_vector(4 downto 0);
signal reg_24_0, reg_24_1 : std_logic_vector(4 downto 0);
signal reg_25_0, reg_25_1 : std_logic_vector(4 downto 0);
signal reg_26_0, reg_26_1 : std_logic_vector(4 downto 0);
signal reg_27_0, reg_27_1 : std_logic_vector(4 downto 0);
signal reg_28_0, reg_28_1 : std_logic_vector(4 downto 0);
signal reg_29_0, reg_29_1 : std_logic_vector(4 downto 0);
signal reg_30_0, reg_30_1 : std_logic_vector(4 downto 0);
signal reg_31_0, reg_31_1 : std_logic_vector(4 downto 0);
signal reg_32_0, reg_32_1 : std_logic_vector(4 downto 0);
signal reg_33_0, reg_33_1 : std_logic_vector(4 downto 0);
signal reg_34_0, reg_34_1 : std_logic_vector(4 downto 0);
signal reg_35_0, reg_35_1 : std_logic_vector(4 downto 0);
signal reg_36_0, reg_36_1 : std_logic_vector(4 downto 0);
signal reg_37_0, reg_37_1 : std_logic_vector(4 downto 0);
signal reg_38_0, reg_38_1 : std_logic_vector(4 downto 0);
signal reg_39_0, reg_39_1 : std_logic_vector(4 downto 0);
signal reg_40_0, reg_40_1 : std_logic_vector(4 downto 0);
signal reg_41_0, reg_41_1 : std_logic_vector(4 downto 0);
signal reg_42_0, reg_42_1 : std_logic_vector(4 downto 0);
signal reg_43_0, reg_43_1 : std_logic_vector(4 downto 0);
signal reg_44_0, reg_44_1 : std_logic_vector(4 downto 0);
signal reg_45_0, reg_45_1 : std_logic_vector(4 downto 0);
signal reg_46_0, reg_46_1 : std_logic_vector(4 downto 0);
signal reg_47_0, reg_47_1 : std_logic_vector(4 downto 0);
signal reg_48_0, reg_48_1 : std_logic_vector(4 downto 0);
signal reg_49_0, reg_49_1 : std_logic_vector(4 downto 0);
signal reg_50_0, reg_50_1 : std_logic_vector(4 downto 0);
signal reg_51_0, reg_51_1 : std_logic_vector(4 downto 0);
signal reg_52_0, reg_52_1 : std_logic_vector(4 downto 0);
signal reg_53_0, reg_53_1 : std_logic_vector(4 downto 0);
signal reg_54_0, reg_54_1 : std_logic_vector(4 downto 0);
signal reg_55_0, reg_55_1 : std_logic_vector(4 downto 0);
signal reg_56_0, reg_56_1 : std_logic_vector(4 downto 0);
signal reg_57_0, reg_57_1 : std_logic_vector(4 downto 0);
signal reg_58_0, reg_58_1 : std_logic_vector(4 downto 0);
signal reg_59_0, reg_59_1 : std_logic_vector(4 downto 0);
signal reg_60_0, reg_60_1 : std_logic_vector(4 downto 0);
signal reg_61_0, reg_61_1 : std_logic_vector(4 downto 0);
signal reg_62_0, reg_62_1 : std_logic_vector(4 downto 0);
signal reg_63_0, reg_63_1 : std_logic_vector(4 downto 0);
signal enb_a, enb_b : std_logic;
signal CO9is1, CO128is0 : std_logic;
signal W4Data : std_logic_vector(383 downto 0);
signal coeff_00, coeff_01, coeff_02, coeff_03, coeff_04, coeff_05, coeff_06, coeff_07, coeff_08, coeff_09: std_logic_vector(2 downto 0);
signal coeff_10, coeff_11, coeff_12, coeff_13, coeff_14, coeff_15, coeff_16, coeff_17, coeff_18, coeff_19: std_logic_vector(2 downto 0);
signal coeff_20, coeff_21, coeff_22, coeff_23, coeff_24, coeff_25, coeff_26, coeff_27, coeff_28, coeff_29: std_logic_vector(2 downto 0);
signal coeff_30, coeff_31, coeff_32, coeff_33, coeff_34, coeff_35, coeff_36, coeff_37, coeff_38, coeff_39: std_logic_vector(2 downto 0);
signal coeff_40, coeff_41, coeff_42, coeff_43, coeff_44, coeff_45, coeff_46, coeff_47, coeff_48, coeff_49: std_logic_vector(2 downto 0);
signal coeff_50, coeff_51, coeff_52, coeff_53, coeff_54, coeff_55, coeff_56, coeff_57, coeff_58, coeff_59: std_logic_vector(2 downto 0);
signal coeff_60, coeff_61, coeff_62, coeff_63: std_logic_vector(2 downto 0);

begin

CO9is1 <= '1' when CO9 = 1 else '0'; 
CO128is0 <= '1' when CO128 = 0 else '0';

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OutChSel_dly2 <= '0';
		OutChSel_dly1 <= '0';
	elsif( CLK'event and CLK = '1' ) then
		OutChSel_dly2 <= OutChSel_dly1;
		OutChSel_dly1 <= OutChSel;
	end if;  
end process;

W4Sel <= OutChSel_dly2;

---------------------------------------------------------
  PixRomData <= 
		X_RD_DATA(	639	downto	635	) when CO128_dly2 = "0000000" else 
		X_RD_DATA(	634	downto	630	) when CO128_dly2 = "0000001" else 
		X_RD_DATA(	629	downto	625	) when CO128_dly2 = "0000010" else 
		X_RD_DATA(	624	downto	620	) when CO128_dly2 = "0000011" else 
		X_RD_DATA(	619	downto	615	) when CO128_dly2 = "0000100" else 
		X_RD_DATA(	614	downto	610	) when CO128_dly2 = "0000101" else 
		X_RD_DATA(	609	downto	605	) when CO128_dly2 = "0000110" else 
		X_RD_DATA(	604	downto	600	) when CO128_dly2 = "0000111" else 
		X_RD_DATA(	599	downto	595	) when CO128_dly2 = "0001000" else 
		X_RD_DATA(	594	downto	590	) when CO128_dly2 = "0001001" else 
		X_RD_DATA(	589	downto	585	) when CO128_dly2 = "0001010" else 
		X_RD_DATA(	584	downto	580	) when CO128_dly2 = "0001011" else 
		X_RD_DATA(	579	downto	575	) when CO128_dly2 = "0001100" else 
		X_RD_DATA(	574	downto	570	) when CO128_dly2 = "0001101" else 
		X_RD_DATA(	569	downto	565	) when CO128_dly2 = "0001110" else 
		X_RD_DATA(	564	downto	560	) when CO128_dly2 = "0001111" else 
		X_RD_DATA(	559	downto	555	) when CO128_dly2 = "0010000" else 
		X_RD_DATA(	554	downto	550	) when CO128_dly2 = "0010001" else 
		X_RD_DATA(	549	downto	545	) when CO128_dly2 = "0010010" else 
		X_RD_DATA(	544	downto	540	) when CO128_dly2 = "0010011" else 
		X_RD_DATA(	539	downto	535	) when CO128_dly2 = "0010100" else 
		X_RD_DATA(	534	downto	530	) when CO128_dly2 = "0010101" else 
		X_RD_DATA(	529	downto	525	) when CO128_dly2 = "0010110" else 
		X_RD_DATA(	524	downto	520	) when CO128_dly2 = "0010111" else 
		X_RD_DATA(	519	downto	515	) when CO128_dly2 = "0011000" else 
		X_RD_DATA(	514	downto	510	) when CO128_dly2 = "0011001" else 
		X_RD_DATA(	509	downto	505	) when CO128_dly2 = "0011010" else 
		X_RD_DATA(	504	downto	500	) when CO128_dly2 = "0011011" else 
		X_RD_DATA(	499	downto	495	) when CO128_dly2 = "0011100" else 
		X_RD_DATA(	494	downto	490	) when CO128_dly2 = "0011101" else 
		X_RD_DATA(	489	downto	485	) when CO128_dly2 = "0011110" else 
		X_RD_DATA(	484	downto	480	) when CO128_dly2 = "0011111" else 
		X_RD_DATA(	479	downto	475	) when CO128_dly2 = "0100000" else 
		X_RD_DATA(	474	downto	470	) when CO128_dly2 = "0100001" else 
		X_RD_DATA(	469	downto	465	) when CO128_dly2 = "0100010" else 
		X_RD_DATA(	464	downto	460	) when CO128_dly2 = "0100011" else 
		X_RD_DATA(	459	downto	455	) when CO128_dly2 = "0100100" else 
		X_RD_DATA(	454	downto	450	) when CO128_dly2 = "0100101" else 
		X_RD_DATA(	449	downto	445	) when CO128_dly2 = "0100110" else 
		X_RD_DATA(	444	downto	440	) when CO128_dly2 = "0100111" else 
		X_RD_DATA(	439	downto	435	) when CO128_dly2 = "0101000" else 
		X_RD_DATA(	434	downto	430	) when CO128_dly2 = "0101001" else 
		X_RD_DATA(	429	downto	425	) when CO128_dly2 = "0101010" else 
		X_RD_DATA(	424	downto	420	) when CO128_dly2 = "0101011" else 
		X_RD_DATA(	419	downto	415	) when CO128_dly2 = "0101100" else 
		X_RD_DATA(	414	downto	410	) when CO128_dly2 = "0101101" else 
		X_RD_DATA(	409	downto	405	) when CO128_dly2 = "0101110" else 
		X_RD_DATA(	404	downto	400	) when CO128_dly2 = "0101111" else 
		X_RD_DATA(	399	downto	395	) when CO128_dly2 = "0110000" else 
		X_RD_DATA(	394	downto	390	) when CO128_dly2 = "0110001" else 
		X_RD_DATA(	389	downto	385	) when CO128_dly2 = "0110010" else 
		X_RD_DATA(	384	downto	380	) when CO128_dly2 = "0110011" else 
		X_RD_DATA(	379	downto	375	) when CO128_dly2 = "0110100" else 
		X_RD_DATA(	374	downto	370	) when CO128_dly2 = "0110101" else 
		X_RD_DATA(	369	downto	365	) when CO128_dly2 = "0110110" else 
		X_RD_DATA(	364	downto	360	) when CO128_dly2 = "0110111" else 
		X_RD_DATA(	359	downto	355	) when CO128_dly2 = "0111000" else 
		X_RD_DATA(	354	downto	350	) when CO128_dly2 = "0111001" else 
		X_RD_DATA(	349	downto	345	) when CO128_dly2 = "0111010" else 
		X_RD_DATA(	344	downto	340	) when CO128_dly2 = "0111011" else 
		X_RD_DATA(	339	downto	335	) when CO128_dly2 = "0111100" else 
		X_RD_DATA(	334	downto	330	) when CO128_dly2 = "0111101" else 
		X_RD_DATA(	329	downto	325	) when CO128_dly2 = "0111110" else 
		X_RD_DATA(	324	downto	320	) when CO128_dly2 = "0111111" else 
		X_RD_DATA(	319	downto	315	) when CO128_dly2 = "1000000" else 
		X_RD_DATA(	314	downto	310	) when CO128_dly2 = "1000001" else 
		X_RD_DATA(	309	downto	305	) when CO128_dly2 = "1000010" else 
		X_RD_DATA(	304	downto	300	) when CO128_dly2 = "1000011" else 
		X_RD_DATA(	299	downto	295	) when CO128_dly2 = "1000100" else 
		X_RD_DATA(	294	downto	290	) when CO128_dly2 = "1000101" else 
		X_RD_DATA(	289	downto	285	) when CO128_dly2 = "1000110" else 
		X_RD_DATA(	284	downto	280	) when CO128_dly2 = "1000111" else 
		X_RD_DATA(	279	downto	275	) when CO128_dly2 = "1001000" else 
		X_RD_DATA(	274	downto	270	) when CO128_dly2 = "1001001" else 
		X_RD_DATA(	269	downto	265	) when CO128_dly2 = "1001010" else 
		X_RD_DATA(	264	downto	260	) when CO128_dly2 = "1001011" else 
		X_RD_DATA(	259	downto	255	) when CO128_dly2 = "1001100" else 
		X_RD_DATA(	254	downto	250	) when CO128_dly2 = "1001101" else 
		X_RD_DATA(	249	downto	245	) when CO128_dly2 = "1001110" else 
		X_RD_DATA(	244	downto	240	) when CO128_dly2 = "1001111" else 
		X_RD_DATA(	239	downto	235	) when CO128_dly2 = "1010000" else 
		X_RD_DATA(	234	downto	230	) when CO128_dly2 = "1010001" else 
		X_RD_DATA(	229	downto	225	) when CO128_dly2 = "1010010" else 
		X_RD_DATA(	224	downto	220	) when CO128_dly2 = "1010011" else 
		X_RD_DATA(	219	downto	215	) when CO128_dly2 = "1010100" else 
		X_RD_DATA(	214	downto	210	) when CO128_dly2 = "1010101" else 
		X_RD_DATA(	209	downto	205	) when CO128_dly2 = "1010110" else 
		X_RD_DATA(	204	downto	200	) when CO128_dly2 = "1010111" else 
		X_RD_DATA(	199	downto	195	) when CO128_dly2 = "1011000" else 
		X_RD_DATA(	194	downto	190	) when CO128_dly2 = "1011001" else 
		X_RD_DATA(	189	downto	185	) when CO128_dly2 = "1011010" else 
		X_RD_DATA(	184	downto	180	) when CO128_dly2 = "1011011" else 
		X_RD_DATA(	179	downto	175	) when CO128_dly2 = "1011100" else 
		X_RD_DATA(	174	downto	170	) when CO128_dly2 = "1011101" else 
		X_RD_DATA(	169	downto	165	) when CO128_dly2 = "1011110" else 
		X_RD_DATA(	164	downto	160	) when CO128_dly2 = "1011111" else 
		X_RD_DATA(	159	downto	155	) when CO128_dly2 = "1100000" else 
		X_RD_DATA(	154	downto	150	) when CO128_dly2 = "1100001" else 
		X_RD_DATA(	149	downto	145	) when CO128_dly2 = "1100010" else 
		X_RD_DATA(	144	downto	140	) when CO128_dly2 = "1100011" else 
		X_RD_DATA(	139	downto	135	) when CO128_dly2 = "1100100" else 
		X_RD_DATA(	134	downto	130	) when CO128_dly2 = "1100101" else 
		X_RD_DATA(	129	downto	125	) when CO128_dly2 = "1100110" else 
		X_RD_DATA(	124	downto	120	) when CO128_dly2 = "1100111" else 
		X_RD_DATA(	119	downto	115	) when CO128_dly2 = "1101000" else 
		X_RD_DATA(	114	downto	110	) when CO128_dly2 = "1101001" else 
		X_RD_DATA(	109	downto	105	) when CO128_dly2 = "1101010" else 
		X_RD_DATA(	104	downto	100	) when CO128_dly2 = "1101011" else 
		X_RD_DATA(	99	downto	95	) when CO128_dly2 = "1101100" else 
		X_RD_DATA(	94	downto	90	) when CO128_dly2 = "1101101" else 
		X_RD_DATA(	89	downto	85	) when CO128_dly2 = "1101110" else 
		X_RD_DATA(	84	downto	80	) when CO128_dly2 = "1101111" else 
		X_RD_DATA(	79	downto	75	) when CO128_dly2 = "1110000" else 
		X_RD_DATA(	74	downto	70	) when CO128_dly2 = "1110001" else 
		X_RD_DATA(	69	downto	65	) when CO128_dly2 = "1110010" else 
		X_RD_DATA(	64	downto	60	) when CO128_dly2 = "1110011" else 
		X_RD_DATA(	59	downto	55	) when CO128_dly2 = "1110100" else 
		X_RD_DATA(	54	downto	50	) when CO128_dly2 = "1110101" else 
		X_RD_DATA(	49	downto	45	) when CO128_dly2 = "1110110" else 
		X_RD_DATA(	44	downto	40	) when CO128_dly2 = "1110111" else 
		X_RD_DATA(	39	downto	35	) when CO128_dly2 = "1111000" else 
		X_RD_DATA(	34	downto	30	) when CO128_dly2 = "1111001" else 
		X_RD_DATA(	29	downto	25	) when CO128_dly2 = "1111010" else 
		X_RD_DATA(	24	downto	20	) when CO128_dly2 = "1111011" else 
		X_RD_DATA(	19	downto	15	) when CO128_dly2 = "1111100" else 
		X_RD_DATA(	14	downto	10	) when CO128_dly2 = "1111101" else 
		X_RD_DATA(	9	downto	5	) when CO128_dly2 = "1111110" else 
		X_RD_DATA(	4	downto	0	); 

W4Addr_L <= CO9;
W4Addr_H <= ("0" & CO128 & "000") + ("0000" & CO128);
W4Addr_prev <= W4Addr_H + ("000000" & W4Addr_L);

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		W4Addr <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if(W4Addr_prev >= -448 and W4Addr_prev <= -1) then -- note that this is Signed
			W4Addr <= (others => '0');
		else
			W4Addr <= W4Addr_prev;
		end if;
	end if;  
end process;

---------------------------------------------------------
enb_a <= '1' when W4Sel = '1' and CO128 = 127 and CO9 = 8 else '0';
enb_b <= '1' when W4Sel = '0' and CO128 = 127 and CO9 = 8 else '0';
--enb_2 <= '1' when W4Sel = 3 and CO128 = 63 else '0';
--enb_3 <= '1' when W4Sel = 0 and CO128 = 63 else '0';

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			outCh_000 <= reg_00_0;
			outCh_001 <= reg_00_1;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_00_0 <= Relu_00;
		elsif(enb_b = '1') then
			reg_00_1 <= Relu_00;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_01_0 <= Relu_01;
		elsif(enb_b = '1') then
			reg_01_1 <= Relu_01;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_02_0 <= Relu_02;
		elsif(enb_b = '1') then
			reg_02_1 <= Relu_02;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_03_0 <= Relu_03;
		elsif(enb_b = '1') then
			reg_03_1 <= Relu_03;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_04_0 <= Relu_04;
		elsif(enb_b = '1') then
			reg_04_1 <= Relu_04;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_05_0 <= Relu_05;
		elsif(enb_b = '1') then
			reg_05_1 <= Relu_05;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_06_0 <= Relu_06;
		elsif(enb_b = '1') then
			reg_06_1 <= Relu_06;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_07_0 <= Relu_07;
		elsif(enb_b = '1') then
			reg_07_1 <= Relu_07;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_08_0 <= Relu_08;
		elsif(enb_b = '1') then
			reg_08_1 <= Relu_08;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_09_0 <= Relu_09;
		elsif(enb_b = '1') then
			reg_09_1 <= Relu_09;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_10_0 <= Relu_10;
		elsif(enb_b = '1') then
			reg_10_1 <= Relu_10;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_11_0 <= Relu_11;
		elsif(enb_b = '1') then
			reg_11_1 <= Relu_11;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_12_0 <= Relu_12;
		elsif(enb_b = '1') then
			reg_12_1 <= Relu_12;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_13_0 <= Relu_13;
		elsif(enb_b = '1') then
			reg_13_1 <= Relu_13;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_14_0 <= Relu_14;
		elsif(enb_b = '1') then
			reg_14_1 <= Relu_14;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_15_0 <= Relu_15;
		elsif(enb_b = '1') then
			reg_15_1 <= Relu_15;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_16_0 <= Relu_16;
		elsif(enb_b = '1') then
			reg_16_1 <= Relu_16;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_17_0 <= Relu_17;
		elsif(enb_b = '1') then
			reg_17_1 <= Relu_17;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_18_0 <= Relu_18;
		elsif(enb_b = '1') then
			reg_18_1 <= Relu_18;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_19_0 <= Relu_19;
		elsif(enb_b = '1') then
			reg_19_1 <= Relu_19;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_20_0 <= Relu_20;
		elsif(enb_b = '1') then
			reg_20_1 <= Relu_20;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_21_0 <= Relu_21;
		elsif(enb_b = '1') then
			reg_21_1 <= Relu_21;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_22_0 <= Relu_22;
		elsif(enb_b = '1') then
			reg_22_1 <= Relu_22;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_23_0 <= Relu_23;
		elsif(enb_b = '1') then
			reg_23_1 <= Relu_23;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_24_0 <= Relu_24;
		elsif(enb_b = '1') then
			reg_24_1 <= Relu_24;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_25_0 <= Relu_25;
		elsif(enb_b = '1') then
			reg_25_1 <= Relu_25;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_26_0 <= Relu_26;
		elsif(enb_b = '1') then
			reg_26_1 <= Relu_26;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_27_0 <= Relu_27;
		elsif(enb_b = '1') then
			reg_27_1 <= Relu_27;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_28_0 <= Relu_28;
		elsif(enb_b = '1') then
			reg_28_1 <= Relu_28;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_29_0 <= Relu_29;
		elsif(enb_b = '1') then
			reg_29_1 <= Relu_29;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_30_0 <= Relu_30;
		elsif(enb_b = '1') then
			reg_30_1 <= Relu_30;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_31_0 <= Relu_31;
		elsif(enb_b = '1') then
			reg_31_1 <= Relu_31;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_32_0 <= Relu_32;
		elsif(enb_b = '1') then
			reg_32_1 <= Relu_32;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_33_0 <= Relu_33;
		elsif(enb_b = '1') then
			reg_33_1 <= Relu_33;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_34_0 <= Relu_34;
		elsif(enb_b = '1') then
			reg_34_1 <= Relu_34;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_35_0 <= Relu_35;
		elsif(enb_b = '1') then
			reg_35_1 <= Relu_35;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_36_0 <= Relu_36;
		elsif(enb_b = '1') then
			reg_36_1 <= Relu_36;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_37_0 <= Relu_37;
		elsif(enb_b = '1') then
			reg_37_1 <= Relu_37;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_38_0 <= Relu_38;
		elsif(enb_b = '1') then
			reg_38_1 <= Relu_38;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_39_0 <= Relu_39;
		elsif(enb_b = '1') then
			reg_39_1 <= Relu_39;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_40_0 <= Relu_40;
		elsif(enb_b = '1') then
			reg_40_1 <= Relu_40;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_41_0 <= Relu_41;
		elsif(enb_b = '1') then
			reg_41_1 <= Relu_41;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_42_0 <= Relu_42;
		elsif(enb_b = '1') then
			reg_42_1 <= Relu_42;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_43_0 <= Relu_43;
		elsif(enb_b = '1') then
			reg_43_1 <= Relu_43;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_44_0 <= Relu_44;
		elsif(enb_b = '1') then
			reg_44_1 <= Relu_44;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_45_0 <= Relu_45;
		elsif(enb_b = '1') then
			reg_45_1 <= Relu_45;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_46_0 <= Relu_46;
		elsif(enb_b = '1') then
			reg_46_1 <= Relu_46;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_47_0 <= Relu_47;
		elsif(enb_b = '1') then
			reg_47_1 <= Relu_47;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_48_0 <= Relu_48;
		elsif(enb_b = '1') then
			reg_48_1 <= Relu_48;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_49_0 <= Relu_49;
		elsif(enb_b = '1') then
			reg_49_1 <= Relu_49;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_50_0 <= Relu_50;
		elsif(enb_b = '1') then
			reg_50_1 <= Relu_50;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_51_0 <= Relu_51;
		elsif(enb_b = '1') then
			reg_51_1 <= Relu_51;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_52_0 <= Relu_52;
		elsif(enb_b = '1') then
			reg_52_1 <= Relu_52;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_53_0 <= Relu_53;
		elsif(enb_b = '1') then
			reg_53_1 <= Relu_53;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_54_0 <= Relu_54;
		elsif(enb_b = '1') then
			reg_54_1 <= Relu_54;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_55_0 <= Relu_55;
		elsif(enb_b = '1') then
			reg_55_1 <= Relu_55;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_56_0 <= Relu_56;
		elsif(enb_b = '1') then
			reg_56_1 <= Relu_56;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_57_0 <= Relu_57;
		elsif(enb_b = '1') then
			reg_57_1 <= Relu_57;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_58_0 <= Relu_58;
		elsif(enb_b = '1') then
			reg_58_1 <= Relu_58;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_59_0 <= Relu_59;
		elsif(enb_b = '1') then
			reg_59_1 <= Relu_59;
		end if;
	end if;  
end process;

-----------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_60_0 <= Relu_60;
		elsif(enb_b = '1') then
			reg_60_1 <= Relu_60;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_61_0 <= Relu_61;
		elsif(enb_b = '1') then
			reg_61_1 <= Relu_61;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_62_0 <= Relu_62;
		elsif(enb_b = '1') then
			reg_62_1 <= Relu_62;
		end if;
	end if;  
end process;

process( RST_N, CLK )  
begin 
	if( CLK'event and CLK = '1' ) then
		if(enb_a = '1') then
			reg_63_0 <= Relu_63;
		elsif(enb_b = '1') then
			reg_63_1 <= Relu_63;
		end if;
	end if;  
end process;

---------------------------------------------------------
ROM_W4_inst : ROM_W4 port map (
  CLK => CLK,
  ADDR => W4Addr,
  DATA => W4Data );

---------------------------------------------------------
coeff_00 <= W4Data( 383 downto 381 ) when W4Sel = '0' else  W4Data( 380 downto 378 );
coeff_01 <= W4Data( 377 downto 375 ) when W4Sel = '0' else  W4Data( 374 downto 372 );
coeff_02 <= W4Data( 371 downto 369 ) when W4Sel = '0' else  W4Data( 368 downto 366 );
coeff_03 <= W4Data( 365 downto 363 ) when W4Sel = '0' else  W4Data( 362 downto 360 );
coeff_04 <= W4Data( 359 downto 357 ) when W4Sel = '0' else  W4Data( 356 downto 354 );
coeff_05 <= W4Data( 353 downto 351 ) when W4Sel = '0' else  W4Data( 350 downto 348 );
coeff_06 <= W4Data( 347 downto 345 ) when W4Sel = '0' else  W4Data( 344 downto 342 );
coeff_07 <= W4Data( 341 downto 339 ) when W4Sel = '0' else  W4Data( 338 downto 336 );
coeff_08 <= W4Data( 335 downto 333 ) when W4Sel = '0' else  W4Data( 332 downto 330 );
coeff_09 <= W4Data( 329 downto 327 ) when W4Sel = '0' else  W4Data( 326 downto 324 );
coeff_10 <= W4Data( 323 downto 321 ) when W4Sel = '0' else  W4Data( 320 downto 318 );
coeff_11 <= W4Data( 317 downto 315 ) when W4Sel = '0' else  W4Data( 314 downto 312 );
coeff_12 <= W4Data( 311 downto 309 ) when W4Sel = '0' else  W4Data( 308 downto 306 );
coeff_13 <= W4Data( 305 downto 303 ) when W4Sel = '0' else  W4Data( 302 downto 300 );
coeff_14 <= W4Data( 299 downto 297 ) when W4Sel = '0' else  W4Data( 296 downto 294 );
coeff_15 <= W4Data( 293 downto 291 ) when W4Sel = '0' else  W4Data( 290 downto 288 );
coeff_16 <= W4Data( 287 downto 285 ) when W4Sel = '0' else  W4Data( 284 downto 282 );
coeff_17 <= W4Data( 281 downto 279 ) when W4Sel = '0' else  W4Data( 278 downto 276 );
coeff_18 <= W4Data( 275 downto 273 ) when W4Sel = '0' else  W4Data( 272 downto 270 );
coeff_19 <= W4Data( 269 downto 267 ) when W4Sel = '0' else  W4Data( 266 downto 264 );
coeff_20 <= W4Data( 263 downto 261 ) when W4Sel = '0' else  W4Data( 260 downto 258 );
coeff_21 <= W4Data( 257 downto 255 ) when W4Sel = '0' else  W4Data( 254 downto 252 );
coeff_22 <= W4Data( 251 downto 249 ) when W4Sel = '0' else  W4Data( 248 downto 246 );
coeff_23 <= W4Data( 245 downto 243 ) when W4Sel = '0' else  W4Data( 242 downto 240 );
coeff_24 <= W4Data( 239 downto 237 ) when W4Sel = '0' else  W4Data( 236 downto 234 );
coeff_25 <= W4Data( 233 downto 231 ) when W4Sel = '0' else  W4Data( 230 downto 228 );
coeff_26 <= W4Data( 227 downto 225 ) when W4Sel = '0' else  W4Data( 224 downto 222 );
coeff_27 <= W4Data( 221 downto 219 ) when W4Sel = '0' else  W4Data( 218 downto 216 );
coeff_28 <= W4Data( 215 downto 213 ) when W4Sel = '0' else  W4Data( 212 downto 210 );
coeff_29 <= W4Data( 209 downto 207 ) when W4Sel = '0' else  W4Data( 206 downto 204 );
coeff_30 <= W4Data( 203 downto 201 ) when W4Sel = '0' else  W4Data( 200 downto 198 );
coeff_31 <= W4Data( 197 downto 195 ) when W4Sel = '0' else  W4Data( 194 downto 192 );
coeff_32 <= W4Data( 191 downto 189 ) when W4Sel = '0' else  W4Data( 188 downto 186 );
coeff_33 <= W4Data( 185 downto 183 ) when W4Sel = '0' else  W4Data( 182 downto 180 );
coeff_34 <= W4Data( 179 downto 177 ) when W4Sel = '0' else  W4Data( 176 downto 174 );
coeff_35 <= W4Data( 173 downto 171 ) when W4Sel = '0' else  W4Data( 170 downto 168 );
coeff_36 <= W4Data( 167 downto 165 ) when W4Sel = '0' else  W4Data( 164 downto 162 );
coeff_37 <= W4Data( 161 downto 159 ) when W4Sel = '0' else  W4Data( 158 downto 156 );
coeff_38 <= W4Data( 155 downto 153 ) when W4Sel = '0' else  W4Data( 152 downto 150 );
coeff_39 <= W4Data( 149 downto 147 ) when W4Sel = '0' else  W4Data( 146 downto 144 );
coeff_40 <= W4Data( 143 downto 141 ) when W4Sel = '0' else  W4Data( 140 downto 138 );
coeff_41 <= W4Data( 137 downto 135 ) when W4Sel = '0' else  W4Data( 134 downto 132 );
coeff_42 <= W4Data( 131 downto 129 ) when W4Sel = '0' else  W4Data( 128 downto 126 );
coeff_43 <= W4Data( 125 downto 123 ) when W4Sel = '0' else  W4Data( 122 downto 120 );
coeff_44 <= W4Data( 119 downto 117 ) when W4Sel = '0' else  W4Data( 116 downto 114 );
coeff_45 <= W4Data( 113 downto 111 ) when W4Sel = '0' else  W4Data( 110 downto 108 );
coeff_46 <= W4Data( 107 downto 105 ) when W4Sel = '0' else  W4Data( 104 downto 102 );
coeff_47 <= W4Data( 101 downto 99 ) when W4Sel = '0' else  W4Data( 98 downto 96 );
coeff_48 <= W4Data( 95 downto 93 ) when W4Sel = '0' else  W4Data( 92 downto 90 );
coeff_49 <= W4Data( 89 downto 87 ) when W4Sel = '0' else  W4Data( 86 downto 84 );
coeff_50 <= W4Data( 83 downto 81 ) when W4Sel = '0' else  W4Data( 80 downto 78 );
coeff_51 <= W4Data( 77 downto 75 ) when W4Sel = '0' else  W4Data( 74 downto 72 );
coeff_52 <= W4Data( 71 downto 69 ) when W4Sel = '0' else  W4Data( 68 downto 66 );
coeff_53 <= W4Data( 65 downto 63 ) when W4Sel = '0' else  W4Data( 62 downto 60 );
coeff_54 <= W4Data( 59 downto 57 ) when W4Sel = '0' else  W4Data( 56 downto 54 );
coeff_55 <= W4Data( 53 downto 51 ) when W4Sel = '0' else  W4Data( 50 downto 48 );
coeff_56 <= W4Data( 47 downto 45 ) when W4Sel = '0' else  W4Data( 44 downto 42 );
coeff_57 <= W4Data( 41 downto 39 ) when W4Sel = '0' else  W4Data( 38 downto 36 );
coeff_58 <= W4Data( 35 downto 33 ) when W4Sel = '0' else  W4Data( 32 downto 30 );
coeff_59 <= W4Data( 29 downto 27 ) when W4Sel = '0' else  W4Data( 26 downto 24 );
coeff_60 <= W4Data( 23 downto 21 ) when W4Sel = '0' else  W4Data( 20 downto 18 );
coeff_61 <= W4Data( 17 downto 15 ) when W4Sel = '0' else  W4Data( 14 downto 12 );
coeff_62 <= W4Data( 11 downto 9 ) when W4Sel = '0' else  W4Data( 8 downto 6 );
coeff_63 <= W4Data( 5 downto 3 ) when W4Sel = '0' else  W4Data( 2 downto 0 );

---------------------------------------------------------
dotpro_W4_00 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_00,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_00 );

dotpro_W4_01 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_01,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_01 );

dotpro_W4_02 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_02,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_02 );

dotpro_W4_03 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_03,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_03 );

dotpro_W4_04 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_04,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_04 );

dotpro_W4_05 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_05,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_05 );

dotpro_W4_06 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_06,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_06 );

dotpro_W4_07 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_07,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_07 );

dotpro_W4_08 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_08,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_08 );

dotpro_W4_09 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_09,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_09 );

---------------------------------------------------------
dotpro_W4_10 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_10,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_10 );

dotpro_W4_11 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_11,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_11 );

dotpro_W4_12 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_12,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_12 );

dotpro_W4_13 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_13,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_13 );

dotpro_W4_14 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_14,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_14 );

dotpro_W4_15 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_15,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_15 );

dotpro_W4_16 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_16,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_16 );

dotpro_W4_17 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_17,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_17 );

dotpro_W4_18 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_18,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_18 );

dotpro_W4_19 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_19,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_19 );

---------------------------------------------------------
dotpro_W4_20 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_20,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_20 );

dotpro_W4_21 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_21,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_21 );

dotpro_W4_22 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_22,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_22 );

dotpro_W4_23 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_23,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_23 );

dotpro_W4_24 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_24,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_24 );

dotpro_W4_25 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_25,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_25 );

dotpro_W4_26 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_26,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_26 );

dotpro_W4_27 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_27,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_27 );

dotpro_W4_28 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_28,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_28 );

dotpro_W4_29 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_29,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_29 );

---------------------------------------------------------
dotpro_W4_30 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_30,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_30 );

dotpro_W4_31 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_31,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_31 );

dotpro_W4_32 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_32,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_32 );

dotpro_W4_33 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_33,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_33 );

dotpro_W4_34 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_34,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_34 );

dotpro_W4_35 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_35,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_35 );

dotpro_W4_36 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_36,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_36 );

dotpro_W4_37 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_37,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_37 );

dotpro_W4_38 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_38,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_38 );

dotpro_W4_39 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_39,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_39 );

---------------------------------------------------------
dotpro_W4_40 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_40,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_40 );

dotpro_W4_41 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_41,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_41 );

dotpro_W4_42 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_42,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_42 );

dotpro_W4_43 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_43,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_43 );

dotpro_W4_44 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_44,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_44 );

dotpro_W4_45 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_45,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_45 );

dotpro_W4_46 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_46,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_46 );

dotpro_W4_47 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_47,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_47 );

dotpro_W4_48 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_48,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_48 );

dotpro_W4_49 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_49,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_49 );

---------------------------------------------------------
dotpro_W4_50 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_50,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_50 );

dotpro_W4_51 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_51,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_51 );

dotpro_W4_52 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_52,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_52 );

dotpro_W4_53 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_53,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_53 );

dotpro_W4_54 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_54,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_54 );

dotpro_W4_55 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_55,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_55 );

dotpro_W4_56 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_56,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_56 );

dotpro_W4_57 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_57,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_57 );

dotpro_W4_58 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_58,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_58 );

dotpro_W4_59 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_59,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_59 );

---------------------------------------------------------
dotpro_W4_60 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_60,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_60 );

dotpro_W4_61 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_61,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_61 );

dotpro_W4_62 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_62,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_62 );

dotpro_W4_63 : dotpro_W4 port map (
  CLK => CLK,
  RST_N => RST_N,
  PixRomData => PixRomData,
  MatRomData => coeff_63,
  CO9is1 => CO9is1,
  CO128is0 => CO128is0,
  MATMULOUT => mac_63 );

---------------------------------------------------------
  truncate_00 <= mac_00(13 downto 7);
  truncate_01 <= mac_01(13 downto 7);
  truncate_02 <= mac_02(13 downto 7);
  truncate_03 <= mac_03(13 downto 7);
  truncate_04 <= mac_04(13 downto 7);
  truncate_05 <= mac_05(13 downto 7);
  truncate_06 <= mac_06(13 downto 7);
  truncate_07 <= mac_07(13 downto 7);
  truncate_08 <= mac_08(13 downto 7);
  truncate_09 <= mac_09(13 downto 7);
  truncate_10 <= mac_10(13 downto 7);
  truncate_11 <= mac_11(13 downto 7);
  truncate_12 <= mac_12(13 downto 7);
  truncate_13 <= mac_13(13 downto 7);
  truncate_14 <= mac_14(13 downto 7);
  truncate_15 <= mac_15(13 downto 7);
  truncate_16 <= mac_16(13 downto 7);
  truncate_17 <= mac_17(13 downto 7);
  truncate_18 <= mac_18(13 downto 7);
  truncate_19 <= mac_19(13 downto 7);
  truncate_20 <= mac_20(13 downto 7);
  truncate_21 <= mac_21(13 downto 7);
  truncate_22 <= mac_22(13 downto 7);
  truncate_23 <= mac_23(13 downto 7);
  truncate_24 <= mac_24(13 downto 7);
  truncate_25 <= mac_25(13 downto 7);
  truncate_26 <= mac_26(13 downto 7);
  truncate_27 <= mac_27(13 downto 7);
  truncate_28 <= mac_28(13 downto 7);
  truncate_29 <= mac_29(13 downto 7);
  truncate_30 <= mac_30(13 downto 7);
  truncate_31 <= mac_31(13 downto 7);
  truncate_32 <= mac_32(13 downto 7);
  truncate_33 <= mac_33(13 downto 7);
  truncate_34 <= mac_34(13 downto 7);
  truncate_35 <= mac_35(13 downto 7);
  truncate_36 <= mac_36(13 downto 7);
  truncate_37 <= mac_37(13 downto 7);
  truncate_38 <= mac_38(13 downto 7);
  truncate_39 <= mac_39(13 downto 7);
  truncate_40 <= mac_40(13 downto 7);
  truncate_41 <= mac_41(13 downto 7);
  truncate_42 <= mac_42(13 downto 7);
  truncate_43 <= mac_43(13 downto 7);
  truncate_44 <= mac_44(13 downto 7);
  truncate_45 <= mac_45(13 downto 7);
  truncate_46 <= mac_46(13 downto 7);
  truncate_47 <= mac_47(13 downto 7);
  truncate_48 <= mac_48(13 downto 7);
  truncate_49 <= mac_49(13 downto 7);
  truncate_50 <= mac_50(13 downto 7);
  truncate_51 <= mac_51(13 downto 7);
  truncate_52 <= mac_52(13 downto 7);
  truncate_53 <= mac_53(13 downto 7);
  truncate_54 <= mac_54(13 downto 7);
  truncate_55 <= mac_55(13 downto 7);
  truncate_56 <= mac_56(13 downto 7);
  truncate_57 <= mac_57(13 downto 7);
  truncate_58 <= mac_58(13 downto 7);
  truncate_59 <= mac_59(13 downto 7);
  truncate_60 <= mac_60(13 downto 7);
  truncate_61 <= mac_61(13 downto 7);
  truncate_62 <= mac_62(13 downto 7);
  truncate_63 <= mac_63(13 downto 7);

---------------------------------------------------------
  Relu_00 <= (others => '0') when truncate_00 < 0 else
             (others => '1') when truncate_00 > 31 else
             truncate_00(4 downto 0);

  Relu_01 <= (others => '0') when truncate_01 < 0 else
             (others => '1') when truncate_01 > 31 else
             truncate_01(4 downto 0);

  Relu_02 <= (others => '0') when truncate_02 < 0 else
             (others => '1') when truncate_02 > 31 else
             truncate_02(4 downto 0);

  Relu_03 <= (others => '0') when truncate_03 < 0 else
             (others => '1') when truncate_03 > 31 else
             truncate_03(4 downto 0);

  Relu_04 <= (others => '0') when truncate_04 < 0 else
             (others => '1') when truncate_04 > 31 else
             truncate_04(4 downto 0);

  Relu_05 <= (others => '0') when truncate_05 < 0 else
             (others => '1') when truncate_05 > 31 else
             truncate_05(4 downto 0);

  Relu_06 <= (others => '0') when truncate_06 < 0 else
             (others => '1') when truncate_06 > 31 else
             truncate_06(4 downto 0);

  Relu_07 <= (others => '0') when truncate_07 < 0 else
             (others => '1') when truncate_07 > 31 else
             truncate_07(4 downto 0);

  Relu_08 <= (others => '0') when truncate_08 < 0 else
             (others => '1') when truncate_08 > 31 else
             truncate_08(4 downto 0);

  Relu_09 <= (others => '0') when truncate_09 < 0 else
             (others => '1') when truncate_09 > 31 else
             truncate_09(4 downto 0);

---------------------------------------------------------
  Relu_10 <= (others => '0') when truncate_10 < 0 else
             (others => '1') when truncate_10 > 31 else
             truncate_10(4 downto 0);

  Relu_11 <= (others => '0') when truncate_11 < 0 else
             (others => '1') when truncate_11 > 31 else
             truncate_11(4 downto 0);

  Relu_12 <= (others => '0') when truncate_12 < 0 else
             (others => '1') when truncate_12 > 31 else
             truncate_12(4 downto 0);

  Relu_13 <= (others => '0') when truncate_13 < 0 else
             (others => '1') when truncate_13 > 31 else
             truncate_13(4 downto 0);

  Relu_14 <= (others => '0') when truncate_14 < 0 else
             (others => '1') when truncate_14 > 31 else
             truncate_14(4 downto 0);

  Relu_15 <= (others => '0') when truncate_15 < 0 else
             (others => '1') when truncate_15 > 31 else
             truncate_15(4 downto 0);

  Relu_16 <= (others => '0') when truncate_16 < 0 else
             (others => '1') when truncate_16 > 31 else
             truncate_16(4 downto 0);

  Relu_17 <= (others => '0') when truncate_17 < 0 else
             (others => '1') when truncate_17 > 31 else
             truncate_17(4 downto 0);

  Relu_18 <= (others => '0') when truncate_18 < 0 else
             (others => '1') when truncate_18 > 31 else
             truncate_18(4 downto 0);

  Relu_19 <= (others => '0') when truncate_19 < 0 else
             (others => '1') when truncate_19 > 31 else
             truncate_19(4 downto 0);

---------------------------------------------------------
  Relu_20 <= (others => '0') when truncate_20 < 0 else
             (others => '1') when truncate_20 > 31 else
             truncate_20(4 downto 0);

  Relu_21 <= (others => '0') when truncate_21 < 0 else
             (others => '1') when truncate_21 > 31 else
             truncate_21(4 downto 0);

  Relu_22 <= (others => '0') when truncate_22 < 0 else
             (others => '1') when truncate_22 > 31 else
             truncate_22(4 downto 0);

  Relu_23 <= (others => '0') when truncate_23 < 0 else
             (others => '1') when truncate_23 > 31 else
             truncate_23(4 downto 0);

  Relu_24 <= (others => '0') when truncate_24 < 0 else
             (others => '1') when truncate_24 > 31 else
             truncate_24(4 downto 0);

  Relu_25 <= (others => '0') when truncate_25 < 0 else
             (others => '1') when truncate_25 > 31 else
             truncate_25(4 downto 0);

  Relu_26 <= (others => '0') when truncate_26 < 0 else
             (others => '1') when truncate_26 > 31 else
             truncate_26(4 downto 0);

  Relu_27 <= (others => '0') when truncate_27 < 0 else
             (others => '1') when truncate_27 > 31 else
             truncate_27(4 downto 0);

  Relu_28 <= (others => '0') when truncate_28 < 0 else
             (others => '1') when truncate_28 > 31 else
             truncate_28(4 downto 0);

  Relu_29 <= (others => '0') when truncate_29 < 0 else
             (others => '1') when truncate_29 > 31 else
             truncate_29(4 downto 0);

---------------------------------------------------------
  Relu_30 <= (others => '0') when truncate_30 < 0 else
             (others => '1') when truncate_30 > 31 else
             truncate_30(4 downto 0);

  Relu_31 <= (others => '0') when truncate_31 < 0 else
             (others => '1') when truncate_31 > 31 else
             truncate_31(4 downto 0);

  Relu_32 <= (others => '0') when truncate_32 < 0 else
             (others => '1') when truncate_32 > 31 else
             truncate_32(4 downto 0);

  Relu_33 <= (others => '0') when truncate_33 < 0 else
             (others => '1') when truncate_33 > 31 else
             truncate_33(4 downto 0);

  Relu_34 <= (others => '0') when truncate_34 < 0 else
             (others => '1') when truncate_34 > 31 else
             truncate_34(4 downto 0);

  Relu_35 <= (others => '0') when truncate_35 < 0 else
             (others => '1') when truncate_35 > 31 else
             truncate_35(4 downto 0);

  Relu_36 <= (others => '0') when truncate_36 < 0 else
             (others => '1') when truncate_36 > 31 else
             truncate_36(4 downto 0);

  Relu_37 <= (others => '0') when truncate_37 < 0 else
             (others => '1') when truncate_37 > 31 else
             truncate_37(4 downto 0);

  Relu_38 <= (others => '0') when truncate_38 < 0 else
             (others => '1') when truncate_38 > 31 else
             truncate_38(4 downto 0);

  Relu_39 <= (others => '0') when truncate_39 < 0 else
             (others => '1') when truncate_39 > 31 else
             truncate_39(4 downto 0);

---------------------------------------------------------
  Relu_40 <= (others => '0') when truncate_40 < 0 else
             (others => '1') when truncate_40 > 31 else
             truncate_40(4 downto 0);

  Relu_41 <= (others => '0') when truncate_41 < 0 else
             (others => '1') when truncate_41 > 31 else
             truncate_41(4 downto 0);

  Relu_42 <= (others => '0') when truncate_42 < 0 else
             (others => '1') when truncate_42 > 31 else
             truncate_42(4 downto 0);

  Relu_43 <= (others => '0') when truncate_43 < 0 else
             (others => '1') when truncate_43 > 31 else
             truncate_43(4 downto 0);

  Relu_44 <= (others => '0') when truncate_44 < 0 else
             (others => '1') when truncate_44 > 31 else
             truncate_44(4 downto 0);

  Relu_45 <= (others => '0') when truncate_45 < 0 else
             (others => '1') when truncate_45 > 31 else
             truncate_45(4 downto 0);

  Relu_46 <= (others => '0') when truncate_46 < 0 else
             (others => '1') when truncate_46 > 31 else
             truncate_46(4 downto 0);

  Relu_47 <= (others => '0') when truncate_47 < 0 else
             (others => '1') when truncate_47 > 31 else
             truncate_47(4 downto 0);

  Relu_48 <= (others => '0') when truncate_48 < 0 else
             (others => '1') when truncate_48 > 31 else
             truncate_48(4 downto 0);

  Relu_49 <= (others => '0') when truncate_49 < 0 else
             (others => '1') when truncate_49 > 31 else
             truncate_49(4 downto 0);

---------------------------------------------------------
  Relu_50 <= (others => '0') when truncate_50 < 0 else
             (others => '1') when truncate_50 > 31 else
             truncate_50(4 downto 0);

  Relu_51 <= (others => '0') when truncate_51 < 0 else
             (others => '1') when truncate_51 > 31 else
             truncate_51(4 downto 0);

  Relu_52 <= (others => '0') when truncate_52 < 0 else
             (others => '1') when truncate_52 > 31 else
             truncate_52(4 downto 0);

  Relu_53 <= (others => '0') when truncate_53 < 0 else
             (others => '1') when truncate_53 > 31 else
             truncate_53(4 downto 0);

  Relu_54 <= (others => '0') when truncate_54 < 0 else
             (others => '1') when truncate_54 > 31 else
             truncate_54(4 downto 0);

  Relu_55 <= (others => '0') when truncate_55 < 0 else
             (others => '1') when truncate_55 > 31 else
             truncate_55(4 downto 0);

  Relu_56 <= (others => '0') when truncate_56 < 0 else
             (others => '1') when truncate_56 > 31 else
             truncate_56(4 downto 0);

  Relu_57 <= (others => '0') when truncate_57 < 0 else
             (others => '1') when truncate_57 > 31 else
             truncate_57(4 downto 0);

  Relu_58 <= (others => '0') when truncate_58 < 0 else
             (others => '1') when truncate_58 > 31 else
             truncate_58(4 downto 0);

  Relu_59 <= (others => '0') when truncate_59 < 0 else
             (others => '1') when truncate_59 > 31 else
             truncate_59(4 downto 0);

---------------------------------------------------------
  Relu_60 <= (others => '0') when truncate_60 < 0 else
             (others => '1') when truncate_60 > 31 else
             truncate_60(4 downto 0);

  Relu_61 <= (others => '0') when truncate_61 < 0 else
             (others => '1') when truncate_61 > 31 else
             truncate_61(4 downto 0);

  Relu_62 <= (others => '0') when truncate_62 < 0 else
             (others => '1') when truncate_62 > 31 else
             truncate_62(4 downto 0);

  Relu_63 <= (others => '0') when truncate_63 < 0 else
             (others => '1') when truncate_63 > 31 else
             truncate_63(4 downto 0);


--ReluSum <= ("0000000" & Relu_00) + ("0000000" & Relu_01)
--		+ ("0000000" & Relu_02) + ("0000000" & Relu_03)
--		+ ("0000000" & Relu_04) + ("0000000" & Relu_05)
--		+ ("0000000" & Relu_06) + ("0000000" & Relu_07)
--		+ ("0000000" & Relu_08) + ("0000000" & Relu_09)
--
--		+ ("0000000" & Relu_10) + ("0000000" & Relu_11)
--		+ ("0000000" & Relu_12) + ("0000000" & Relu_13)
--		+ ("0000000" & Relu_14) + ("0000000" & Relu_15)
--		+ ("0000000" & Relu_16) + ("0000000" & Relu_17)
--		+ ("0000000" & Relu_18) + ("0000000" & Relu_19)
--
--		+ ("0000000" & Relu_20) + ("0000000" & Relu_21)
--		+ ("0000000" & Relu_22) + ("0000000" & Relu_23)
--		+ ("0000000" & Relu_24) + ("0000000" & Relu_25)
--		+ ("0000000" & Relu_26) + ("0000000" & Relu_27)
--		+ ("0000000" & Relu_28) + ("0000000" & Relu_29)
--
--		+ ("0000000" & Relu_30) + ("0000000" & Relu_31);

ReluAll <= 
	reg_00_0 & reg_00_1 & 
	reg_01_0 & reg_01_1 & 
	reg_02_0 & reg_02_1 & 
	reg_03_0 & reg_03_1 & 
	reg_04_0 & reg_04_1 & 
	reg_05_0 & reg_05_1 & 
	reg_06_0 & reg_06_1 & 
	reg_07_0 & reg_07_1 & 
	reg_08_0 & reg_08_1 & 
	reg_09_0 & reg_09_1 & 
	reg_10_0 & reg_10_1 & 
	reg_11_0 & reg_11_1 & 
	reg_12_0 & reg_12_1 & 
	reg_13_0 & reg_13_1 & 
	reg_14_0 & reg_14_1 & 
	reg_15_0 & reg_15_1 & 
	reg_16_0 & reg_16_1 & 
	reg_17_0 & reg_17_1 & 
	reg_18_0 & reg_18_1 & 
	reg_19_0 & reg_19_1 & 
	reg_20_0 & reg_20_1 & 
	reg_21_0 & reg_21_1 & 
	reg_22_0 & reg_22_1 & 
	reg_23_0 & reg_23_1 & 
	reg_24_0 & reg_24_1 & 
	reg_25_0 & reg_25_1 & 
	reg_26_0 & reg_26_1 & 
	reg_27_0 & reg_27_1 & 
	reg_28_0 & reg_28_1 & 
	reg_29_0 & reg_29_1 & 
	reg_30_0 & reg_30_1 & 
	reg_31_0 & reg_31_1 & 
	reg_32_0 & reg_32_1 & 
	reg_33_0 & reg_33_1 & 
	reg_34_0 & reg_34_1 & 
	reg_35_0 & reg_35_1 & 
	reg_36_0 & reg_36_1 & 
	reg_37_0 & reg_37_1 & 
	reg_38_0 & reg_38_1 & 
	reg_39_0 & reg_39_1 & 
	reg_40_0 & reg_40_1 & 
	reg_41_0 & reg_41_1 & 
	reg_42_0 & reg_42_1 & 
	reg_43_0 & reg_43_1 & 
	reg_44_0 & reg_44_1 & 
	reg_45_0 & reg_45_1 & 
	reg_46_0 & reg_46_1 & 
	reg_47_0 & reg_47_1 & 
	reg_48_0 & reg_48_1 & 
	reg_49_0 & reg_49_1 & 
	reg_50_0 & reg_50_1 & 
	reg_51_0 & reg_51_1 & 
	reg_52_0 & reg_52_1 & 
	reg_53_0 & reg_53_1 & 
	reg_54_0 & reg_54_1 & 
	reg_55_0 & reg_55_1 & 
	reg_56_0 & reg_56_1 & 
	reg_57_0 & reg_57_1 & 
	reg_58_0 & reg_58_1 & 
	reg_59_0 & reg_59_1 & 
	reg_60_0 & reg_60_1 & 
	reg_61_0 & reg_61_1 & 
	reg_62_0 & reg_62_1 & 
	reg_63_0 & reg_63_1 ; 

ReluAllWrN <= not enb_a;

end RTL;

