-- A module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2020/02/23

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_signed.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity pooling_lay4 is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  ReluAll : in std_logic_vector(639 downto 0); 
  WR_N : in std_logic; 
  HCO150 : in std_logic_vector(7 downto 0); 
  Poolout : Out std_logic_vector(639 downto 0) );  
end pooling_lay4;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of pooling_lay4 is

component SRAM_256x640_pool
Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(7 downto 0);
          RDADDR : In std_logic_vector(7 downto 0);
          WRDATA : In std_logic_vector(639 downto 0);
          RDDATA : Out std_logic_vector(639 downto 0) );
end component;

component pooling_max
Port (
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  Relu0 : in std_logic_vector(4 downto 0); 
  Relu1 : in std_logic_vector(4 downto 0); 
  WR_N : in std_logic; 
  HCO300 : in std_logic_vector(8 downto 0); 
  Poolout : Out std_logic_vector(4 downto 0) ); 
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal befdly_000,befdly_001,befdly_002,befdly_003,befdly_004,befdly_005,befdly_006,befdly_007,befdly_008,befdly_009 : std_logic_vector(4 downto 0);
signal befdly_010,befdly_011,befdly_012,befdly_013,befdly_014,befdly_015,befdly_016,befdly_017,befdly_018,befdly_019 : std_logic_vector(4 downto 0);
signal befdly_020,befdly_021,befdly_022,befdly_023,befdly_024,befdly_025,befdly_026,befdly_027,befdly_028,befdly_029 : std_logic_vector(4 downto 0);
signal befdly_030,befdly_031,befdly_032,befdly_033,befdly_034,befdly_035,befdly_036,befdly_037,befdly_038,befdly_039 : std_logic_vector(4 downto 0);
signal befdly_040,befdly_041,befdly_042,befdly_043,befdly_044,befdly_045,befdly_046,befdly_047,befdly_048,befdly_049 : std_logic_vector(4 downto 0);
signal befdly_050,befdly_051,befdly_052,befdly_053,befdly_054,befdly_055,befdly_056,befdly_057,befdly_058,befdly_059 : std_logic_vector(4 downto 0);
signal befdly_060,befdly_061,befdly_062,befdly_063,befdly_064,befdly_065,befdly_066,befdly_067,befdly_068,befdly_069 : std_logic_vector(4 downto 0);
signal befdly_070,befdly_071,befdly_072,befdly_073,befdly_074,befdly_075,befdly_076,befdly_077,befdly_078,befdly_079 : std_logic_vector(4 downto 0);
signal befdly_080,befdly_081,befdly_082,befdly_083,befdly_084,befdly_085,befdly_086,befdly_087,befdly_088,befdly_089 : std_logic_vector(4 downto 0);
signal befdly_090,befdly_091,befdly_092,befdly_093,befdly_094,befdly_095,befdly_096,befdly_097,befdly_098,befdly_099 : std_logic_vector(4 downto 0);
signal befdly_100,befdly_101,befdly_102,befdly_103,befdly_104,befdly_105,befdly_106,befdly_107,befdly_108,befdly_109 : std_logic_vector(4 downto 0);
signal befdly_110,befdly_111,befdly_112,befdly_113,befdly_114,befdly_115,befdly_116,befdly_117,befdly_118,befdly_119 : std_logic_vector(4 downto 0);
signal befdly_120,befdly_121,befdly_122,befdly_123,befdly_124,befdly_125,befdly_126,befdly_127 : std_logic_vector(4 downto 0);

signal aftdly_000,aftdly_001,aftdly_002,aftdly_003,aftdly_004,aftdly_005,aftdly_006,aftdly_007,aftdly_008,aftdly_009 : std_logic_vector(4 downto 0);
signal aftdly_010,aftdly_011,aftdly_012,aftdly_013,aftdly_014,aftdly_015,aftdly_016,aftdly_017,aftdly_018,aftdly_019 : std_logic_vector(4 downto 0);
signal aftdly_020,aftdly_021,aftdly_022,aftdly_023,aftdly_024,aftdly_025,aftdly_026,aftdly_027,aftdly_028,aftdly_029 : std_logic_vector(4 downto 0);
signal aftdly_030,aftdly_031,aftdly_032,aftdly_033,aftdly_034,aftdly_035,aftdly_036,aftdly_037,aftdly_038,aftdly_039 : std_logic_vector(4 downto 0);
signal aftdly_040,aftdly_041,aftdly_042,aftdly_043,aftdly_044,aftdly_045,aftdly_046,aftdly_047,aftdly_048,aftdly_049 : std_logic_vector(4 downto 0);
signal aftdly_050,aftdly_051,aftdly_052,aftdly_053,aftdly_054,aftdly_055,aftdly_056,aftdly_057,aftdly_058,aftdly_059 : std_logic_vector(4 downto 0);
signal aftdly_060,aftdly_061,aftdly_062,aftdly_063,aftdly_064,aftdly_065,aftdly_066,aftdly_067,aftdly_068,aftdly_069 : std_logic_vector(4 downto 0);
signal aftdly_070,aftdly_071,aftdly_072,aftdly_073,aftdly_074,aftdly_075,aftdly_076,aftdly_077,aftdly_078,aftdly_079 : std_logic_vector(4 downto 0);
signal aftdly_080,aftdly_081,aftdly_082,aftdly_083,aftdly_084,aftdly_085,aftdly_086,aftdly_087,aftdly_088,aftdly_089 : std_logic_vector(4 downto 0);
signal aftdly_090,aftdly_091,aftdly_092,aftdly_093,aftdly_094,aftdly_095,aftdly_096,aftdly_097,aftdly_098,aftdly_099 : std_logic_vector(4 downto 0);
signal aftdly_100,aftdly_101,aftdly_102,aftdly_103,aftdly_104,aftdly_105,aftdly_106,aftdly_107,aftdly_108,aftdly_109 : std_logic_vector(4 downto 0);
signal aftdly_110,aftdly_111,aftdly_112,aftdly_113,aftdly_114,aftdly_115,aftdly_116,aftdly_117,aftdly_118,aftdly_119 : std_logic_vector(4 downto 0);
signal aftdly_120,aftdly_121,aftdly_122,aftdly_123,aftdly_124,aftdly_125,aftdly_126,aftdly_127 : std_logic_vector(4 downto 0);

signal pool_000,pool_001,pool_002,pool_003,pool_004,pool_005,pool_006,pool_007,pool_008,pool_009 : std_logic_vector(4 downto 0);
signal pool_010,pool_011,pool_012,pool_013,pool_014,pool_015,pool_016,pool_017,pool_018,pool_019 : std_logic_vector(4 downto 0);
signal pool_020,pool_021,pool_022,pool_023,pool_024,pool_025,pool_026,pool_027,pool_028,pool_029 : std_logic_vector(4 downto 0);
signal pool_030,pool_031,pool_032,pool_033,pool_034,pool_035,pool_036,pool_037,pool_038,pool_039 : std_logic_vector(4 downto 0);
signal pool_040,pool_041,pool_042,pool_043,pool_044,pool_045,pool_046,pool_047,pool_048,pool_049 : std_logic_vector(4 downto 0);
signal pool_050,pool_051,pool_052,pool_053,pool_054,pool_055,pool_056,pool_057,pool_058,pool_059 : std_logic_vector(4 downto 0);
signal pool_060,pool_061,pool_062,pool_063,pool_064,pool_065,pool_066,pool_067,pool_068,pool_069 : std_logic_vector(4 downto 0);
signal pool_070,pool_071,pool_072,pool_073,pool_074,pool_075,pool_076,pool_077,pool_078,pool_079 : std_logic_vector(4 downto 0);
signal pool_080,pool_081,pool_082,pool_083,pool_084,pool_085,pool_086,pool_087,pool_088,pool_089 : std_logic_vector(4 downto 0);
signal pool_090,pool_091,pool_092,pool_093,pool_094,pool_095,pool_096,pool_097,pool_098,pool_099 : std_logic_vector(4 downto 0);
signal pool_100,pool_101,pool_102,pool_103,pool_104,pool_105,pool_106,pool_107,pool_108,pool_109 : std_logic_vector(4 downto 0);
signal pool_110,pool_111,pool_112,pool_113,pool_114,pool_115,pool_116,pool_117,pool_118,pool_119 : std_logic_vector(4 downto 0);
signal pool_120,pool_121,pool_122,pool_123,pool_124,pool_125,pool_126,pool_127 : std_logic_vector(4 downto 0);

signal ReluAllDly : std_logic_vector(639 downto 0);
signal HCO300 : std_logic_vector(8 downto 0);

begin
---------------------------------------------------------
RAM_P2_0 : SRAM_256x640_pool port map (
   WRCLK	=> CLK,
   RDCLK	=> CLK,
   WR_N		=> WR_N, 
   WRADDR	=> HCO150, 
   RDADDR	=> HCO150, 
   WRDATA	=> ReluAll, 
   RDDATA	=> ReluAllDly);

---------------------------------------------------------
HCO300 <= '0' & HCO150;

---------------------------------------------------------
pooling_max_000 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_000, 
   Relu1	=> aftdly_000, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_000);

pooling_max_001 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_001, 
   Relu1	=> aftdly_001, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_001);

pooling_max_002 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_002, 
   Relu1	=> aftdly_002, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_002);

pooling_max_003 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_003, 
   Relu1	=> aftdly_003, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_003);

pooling_max_004 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_004, 
   Relu1	=> aftdly_004, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_004);

pooling_max_005 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_005, 
   Relu1	=> aftdly_005, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_005);

pooling_max_006 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_006, 
   Relu1	=> aftdly_006, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_006);

pooling_max_007 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_007, 
   Relu1	=> aftdly_007, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_007);

pooling_max_008 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_008, 
   Relu1	=> aftdly_008, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_008);

pooling_max_009 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_009, 
   Relu1	=> aftdly_009, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_009);

---------------------------------------------------------
pooling_max_010 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_010, 
   Relu1	=> aftdly_010, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_010);

pooling_max_011 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_011, 
   Relu1	=> aftdly_011, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_011);

pooling_max_012 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_012, 
   Relu1	=> aftdly_012, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_012);

pooling_max_013 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_013, 
   Relu1	=> aftdly_013, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_013);

pooling_max_014 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_014, 
   Relu1	=> aftdly_014, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_014);

pooling_max_015 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_015, 
   Relu1	=> aftdly_015, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_015);

pooling_max_016 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_016, 
   Relu1	=> aftdly_016, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_016);

pooling_max_017 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_017, 
   Relu1	=> aftdly_017, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_017);

pooling_max_018 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_018, 
   Relu1	=> aftdly_018, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_018);

pooling_max_019 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_019, 
   Relu1	=> aftdly_019, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_019);

---------------------------------------------------------
pooling_max_020 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_020, 
   Relu1	=> aftdly_020, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_020);

pooling_max_021 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_021, 
   Relu1	=> aftdly_021, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_021);

pooling_max_022 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_022, 
   Relu1	=> aftdly_022, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_022);

pooling_max_023 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_023, 
   Relu1	=> aftdly_023, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_023);

pooling_max_024 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_024, 
   Relu1	=> aftdly_024, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_024);

pooling_max_025 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_025, 
   Relu1	=> aftdly_025, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_025);

pooling_max_026 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_026, 
   Relu1	=> aftdly_026, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_026);

pooling_max_027 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_027, 
   Relu1	=> aftdly_027, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_027);

pooling_max_028 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_028, 
   Relu1	=> aftdly_028, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_028);

pooling_max_029 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_029, 
   Relu1	=> aftdly_029, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_029);

---------------------------------------------------------
pooling_max_030 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_030, 
   Relu1	=> aftdly_030, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_030);

pooling_max_031 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_031, 
   Relu1	=> aftdly_031, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_031);

pooling_max_032 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_032, 
   Relu1	=> aftdly_032, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_032);

pooling_max_033 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_033, 
   Relu1	=> aftdly_033, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_033);

pooling_max_034 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_034, 
   Relu1	=> aftdly_034, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_034);

pooling_max_035 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_035, 
   Relu1	=> aftdly_035, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_035);

pooling_max_036 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_036, 
   Relu1	=> aftdly_036, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_036);

pooling_max_037 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_037, 
   Relu1	=> aftdly_037, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_037);

pooling_max_038 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_038, 
   Relu1	=> aftdly_038, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_038);

pooling_max_039 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_039, 
   Relu1	=> aftdly_039, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_039);

---------------------------------------------------------
pooling_max_040 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_040, 
   Relu1	=> aftdly_040, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_040);

pooling_max_041 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_041, 
   Relu1	=> aftdly_041, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_041);

pooling_max_042 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_042, 
   Relu1	=> aftdly_042, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_042);

pooling_max_043 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_043, 
   Relu1	=> aftdly_043, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_043);

pooling_max_044 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_044, 
   Relu1	=> aftdly_044, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_044);

pooling_max_045 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_045, 
   Relu1	=> aftdly_045, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_045);

pooling_max_046 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_046, 
   Relu1	=> aftdly_046, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_046);

pooling_max_047 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_047, 
   Relu1	=> aftdly_047, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_047);

pooling_max_048 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_048, 
   Relu1	=> aftdly_048, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_048);

pooling_max_049 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_049, 
   Relu1	=> aftdly_049, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_049);

---------------------------------------------------------
pooling_max_050 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_050, 
   Relu1	=> aftdly_050, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_050);

pooling_max_051 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_051, 
   Relu1	=> aftdly_051, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_051);

pooling_max_052 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_052, 
   Relu1	=> aftdly_052, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_052);

pooling_max_053 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_053, 
   Relu1	=> aftdly_053, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_053);

pooling_max_054 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_054, 
   Relu1	=> aftdly_054, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_054);

pooling_max_055 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_055, 
   Relu1	=> aftdly_055, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_055);

pooling_max_056 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_056, 
   Relu1	=> aftdly_056, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_056);

pooling_max_057 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_057, 
   Relu1	=> aftdly_057, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_057);

pooling_max_058 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_058, 
   Relu1	=> aftdly_058, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_058);

pooling_max_059 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_059, 
   Relu1	=> aftdly_059, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_059);

---------------------------------------------------------
pooling_max_060 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_060, 
   Relu1	=> aftdly_060, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_060);

pooling_max_061 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_061, 
   Relu1	=> aftdly_061, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_061);

pooling_max_062 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_062, 
   Relu1	=> aftdly_062, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_062);

pooling_max_063 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_063, 
   Relu1	=> aftdly_063, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_063);

pooling_max_064 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_064, 
   Relu1	=> aftdly_064, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_064);

pooling_max_065 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_065, 
   Relu1	=> aftdly_065, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_065);

pooling_max_066 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_066, 
   Relu1	=> aftdly_066, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_066);

pooling_max_067 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_067, 
   Relu1	=> aftdly_067, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_067);

pooling_max_068 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_068, 
   Relu1	=> aftdly_068, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_068);

pooling_max_069 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_069, 
   Relu1	=> aftdly_069, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_069);

---------------------------------------------------------
pooling_max_070 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_070, 
   Relu1	=> aftdly_070, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_070);

pooling_max_071 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_071, 
   Relu1	=> aftdly_071, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_071);

pooling_max_072 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_072, 
   Relu1	=> aftdly_072, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_072);

pooling_max_073 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_073, 
   Relu1	=> aftdly_073, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_073);

pooling_max_074 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_074, 
   Relu1	=> aftdly_074, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_074);

pooling_max_075 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_075, 
   Relu1	=> aftdly_075, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_075);

pooling_max_076 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_076, 
   Relu1	=> aftdly_076, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_076);

pooling_max_077 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_077, 
   Relu1	=> aftdly_077, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_077);

pooling_max_078 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_078, 
   Relu1	=> aftdly_078, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_078);

pooling_max_079 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_079, 
   Relu1	=> aftdly_079, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_079);

---------------------------------------------------------
pooling_max_080 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_080, 
   Relu1	=> aftdly_080, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_080);

pooling_max_081 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_081, 
   Relu1	=> aftdly_081, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_081);

pooling_max_082 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_082, 
   Relu1	=> aftdly_082, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_082);

pooling_max_083 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_083, 
   Relu1	=> aftdly_083, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_083);

pooling_max_084 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_084, 
   Relu1	=> aftdly_084, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_084);

pooling_max_085 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_085, 
   Relu1	=> aftdly_085, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_085);

pooling_max_086 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_086, 
   Relu1	=> aftdly_086, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_086);

pooling_max_087 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_087, 
   Relu1	=> aftdly_087, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_087);

pooling_max_088 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_088, 
   Relu1	=> aftdly_088, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_088);

pooling_max_089 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_089, 
   Relu1	=> aftdly_089, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_089);

---------------------------------------------------------
pooling_max_090 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_090, 
   Relu1	=> aftdly_090, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_090);

pooling_max_091 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_091, 
   Relu1	=> aftdly_091, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_091);

pooling_max_092 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_092, 
   Relu1	=> aftdly_092, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_092);

pooling_max_093 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_093, 
   Relu1	=> aftdly_093, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_093);

pooling_max_094 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_094, 
   Relu1	=> aftdly_094, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_094);

pooling_max_095 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_095, 
   Relu1	=> aftdly_095, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_095);

pooling_max_096 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_096, 
   Relu1	=> aftdly_096, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_096);

pooling_max_097 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_097, 
   Relu1	=> aftdly_097, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_097);

pooling_max_098 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_098, 
   Relu1	=> aftdly_098, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_098);

pooling_max_099 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_099, 
   Relu1	=> aftdly_099, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_099);

---------------------------------------------------------
pooling_max_100 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_100, 
   Relu1	=> aftdly_100, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_100);

pooling_max_101 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_101, 
   Relu1	=> aftdly_101, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_101);

pooling_max_102 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_102, 
   Relu1	=> aftdly_102, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_102);

pooling_max_103 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_103, 
   Relu1	=> aftdly_103, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_103);

pooling_max_104 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_104, 
   Relu1	=> aftdly_104, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_104);

pooling_max_105 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_105, 
   Relu1	=> aftdly_105, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_105);

pooling_max_106 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_106, 
   Relu1	=> aftdly_106, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_106);

pooling_max_107 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_107, 
   Relu1	=> aftdly_107, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_107);

pooling_max_108 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_108, 
   Relu1	=> aftdly_108, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_108);

pooling_max_109 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_109, 
   Relu1	=> aftdly_109, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_109);

---------------------------------------------------------
pooling_max_110 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_110, 
   Relu1	=> aftdly_110, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_110);

pooling_max_111 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_111, 
   Relu1	=> aftdly_111, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_111);

pooling_max_112 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_112, 
   Relu1	=> aftdly_112, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_112);

pooling_max_113 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_113, 
   Relu1	=> aftdly_113, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_113);

pooling_max_114 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_114, 
   Relu1	=> aftdly_114, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_114);

pooling_max_115 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_115, 
   Relu1	=> aftdly_115, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_115);

pooling_max_116 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_116, 
   Relu1	=> aftdly_116, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_116);

pooling_max_117 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_117, 
   Relu1	=> aftdly_117, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_117);

pooling_max_118 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_118, 
   Relu1	=> aftdly_118, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_118);

pooling_max_119 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_119, 
   Relu1	=> aftdly_119, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_119);

---------------------------------------------------------
pooling_max_120 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_120, 
   Relu1	=> aftdly_120, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_120);

pooling_max_121 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_121, 
   Relu1	=> aftdly_121, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_121);

pooling_max_122 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_122, 
   Relu1	=> aftdly_122, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_122);

pooling_max_123 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_123, 
   Relu1	=> aftdly_123, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_123);

pooling_max_124 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_124, 
   Relu1	=> aftdly_124, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_124);

pooling_max_125 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_125, 
   Relu1	=> aftdly_125, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_125);

pooling_max_126 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_126, 
   Relu1	=> aftdly_126, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_126);

pooling_max_127 : pooling_max port map (
   CLK	=> CLK,
   RST_N	=> RST_N,
   Relu0	=> befdly_127, 
   Relu1	=> aftdly_127, 
   WR_N		=> WR_N, 
   HCO300	=> HCO300, 
   Poolout	=> pool_127);


befdly_000	<=	ReluAll (	639	downto	635	);
befdly_001	<=	ReluAll (	634	downto	630	);
befdly_002	<=	ReluAll (	629	downto	625	);
befdly_003	<=	ReluAll (	624	downto	620	);
befdly_004	<=	ReluAll (	619	downto	615	);
befdly_005	<=	ReluAll (	614	downto	610	);
befdly_006	<=	ReluAll (	609	downto	605	);
befdly_007	<=	ReluAll (	604	downto	600	);
befdly_008	<=	ReluAll (	599	downto	595	);
befdly_009	<=	ReluAll (	594	downto	590	);
befdly_010	<=	ReluAll (	589	downto	585	);
befdly_011	<=	ReluAll (	584	downto	580	);
befdly_012	<=	ReluAll (	579	downto	575	);
befdly_013	<=	ReluAll (	574	downto	570	);
befdly_014	<=	ReluAll (	569	downto	565	);
befdly_015	<=	ReluAll (	564	downto	560	);
befdly_016	<=	ReluAll (	559	downto	555	);
befdly_017	<=	ReluAll (	554	downto	550	);
befdly_018	<=	ReluAll (	549	downto	545	);
befdly_019	<=	ReluAll (	544	downto	540	);
befdly_020	<=	ReluAll (	539	downto	535	);
befdly_021	<=	ReluAll (	534	downto	530	);
befdly_022	<=	ReluAll (	529	downto	525	);
befdly_023	<=	ReluAll (	524	downto	520	);
befdly_024	<=	ReluAll (	519	downto	515	);
befdly_025	<=	ReluAll (	514	downto	510	);
befdly_026	<=	ReluAll (	509	downto	505	);
befdly_027	<=	ReluAll (	504	downto	500	);
befdly_028	<=	ReluAll (	499	downto	495	);
befdly_029	<=	ReluAll (	494	downto	490	);
befdly_030	<=	ReluAll (	489	downto	485	);
befdly_031	<=	ReluAll (	484	downto	480	);
befdly_032	<=	ReluAll (	479	downto	475	);
befdly_033	<=	ReluAll (	474	downto	470	);
befdly_034	<=	ReluAll (	469	downto	465	);
befdly_035	<=	ReluAll (	464	downto	460	);
befdly_036	<=	ReluAll (	459	downto	455	);
befdly_037	<=	ReluAll (	454	downto	450	);
befdly_038	<=	ReluAll (	449	downto	445	);
befdly_039	<=	ReluAll (	444	downto	440	);
befdly_040	<=	ReluAll (	439	downto	435	);
befdly_041	<=	ReluAll (	434	downto	430	);
befdly_042	<=	ReluAll (	429	downto	425	);
befdly_043	<=	ReluAll (	424	downto	420	);
befdly_044	<=	ReluAll (	419	downto	415	);
befdly_045	<=	ReluAll (	414	downto	410	);
befdly_046	<=	ReluAll (	409	downto	405	);
befdly_047	<=	ReluAll (	404	downto	400	);
befdly_048	<=	ReluAll (	399	downto	395	);
befdly_049	<=	ReluAll (	394	downto	390	);
befdly_050	<=	ReluAll (	389	downto	385	);
befdly_051	<=	ReluAll (	384	downto	380	);
befdly_052	<=	ReluAll (	379	downto	375	);
befdly_053	<=	ReluAll (	374	downto	370	);
befdly_054	<=	ReluAll (	369	downto	365	);
befdly_055	<=	ReluAll (	364	downto	360	);
befdly_056	<=	ReluAll (	359	downto	355	);
befdly_057	<=	ReluAll (	354	downto	350	);
befdly_058	<=	ReluAll (	349	downto	345	);
befdly_059	<=	ReluAll (	344	downto	340	);
befdly_060	<=	ReluAll (	339	downto	335	);
befdly_061	<=	ReluAll (	334	downto	330	);
befdly_062	<=	ReluAll (	329	downto	325	);
befdly_063	<=	ReluAll (	324	downto	320	);
befdly_064	<=	ReluAll (	319	downto	315	);
befdly_065	<=	ReluAll (	314	downto	310	);
befdly_066	<=	ReluAll (	309	downto	305	);
befdly_067	<=	ReluAll (	304	downto	300	);
befdly_068	<=	ReluAll (	299	downto	295	);
befdly_069	<=	ReluAll (	294	downto	290	);
befdly_070	<=	ReluAll (	289	downto	285	);
befdly_071	<=	ReluAll (	284	downto	280	);
befdly_072	<=	ReluAll (	279	downto	275	);
befdly_073	<=	ReluAll (	274	downto	270	);
befdly_074	<=	ReluAll (	269	downto	265	);
befdly_075	<=	ReluAll (	264	downto	260	);
befdly_076	<=	ReluAll (	259	downto	255	);
befdly_077	<=	ReluAll (	254	downto	250	);
befdly_078	<=	ReluAll (	249	downto	245	);
befdly_079	<=	ReluAll (	244	downto	240	);
befdly_080	<=	ReluAll (	239	downto	235	);
befdly_081	<=	ReluAll (	234	downto	230	);
befdly_082	<=	ReluAll (	229	downto	225	);
befdly_083	<=	ReluAll (	224	downto	220	);
befdly_084	<=	ReluAll (	219	downto	215	);
befdly_085	<=	ReluAll (	214	downto	210	);
befdly_086	<=	ReluAll (	209	downto	205	);
befdly_087	<=	ReluAll (	204	downto	200	);
befdly_088	<=	ReluAll (	199	downto	195	);
befdly_089	<=	ReluAll (	194	downto	190	);
befdly_090	<=	ReluAll (	189	downto	185	);
befdly_091	<=	ReluAll (	184	downto	180	);
befdly_092	<=	ReluAll (	179	downto	175	);
befdly_093	<=	ReluAll (	174	downto	170	);
befdly_094	<=	ReluAll (	169	downto	165	);
befdly_095	<=	ReluAll (	164	downto	160	);
befdly_096	<=	ReluAll (	159	downto	155	);
befdly_097	<=	ReluAll (	154	downto	150	);
befdly_098	<=	ReluAll (	149	downto	145	);
befdly_099	<=	ReluAll (	144	downto	140	);
befdly_100	<=	ReluAll (	139	downto	135	);
befdly_101	<=	ReluAll (	134	downto	130	);
befdly_102	<=	ReluAll (	129	downto	125	);
befdly_103	<=	ReluAll (	124	downto	120	);
befdly_104	<=	ReluAll (	119	downto	115	);
befdly_105	<=	ReluAll (	114	downto	110	);
befdly_106	<=	ReluAll (	109	downto	105	);
befdly_107	<=	ReluAll (	104	downto	100	);
befdly_108	<=	ReluAll (	99	downto	95	);
befdly_109	<=	ReluAll (	94	downto	90	);
befdly_110	<=	ReluAll (	89	downto	85	);
befdly_111	<=	ReluAll (	84	downto	80	);
befdly_112	<=	ReluAll (	79	downto	75	);
befdly_113	<=	ReluAll (	74	downto	70	);
befdly_114	<=	ReluAll (	69	downto	65	);
befdly_115	<=	ReluAll (	64	downto	60	);
befdly_116	<=	ReluAll (	59	downto	55	);
befdly_117	<=	ReluAll (	54	downto	50	);
befdly_118	<=	ReluAll (	49	downto	45	);
befdly_119	<=	ReluAll (	44	downto	40	);
befdly_120	<=	ReluAll (	39	downto	35	);
befdly_121	<=	ReluAll (	34	downto	30	);
befdly_122	<=	ReluAll (	29	downto	25	);
befdly_123	<=	ReluAll (	24	downto	20	);
befdly_124	<=	ReluAll (	19	downto	15	);
befdly_125	<=	ReluAll (	14	downto	10	);
befdly_126	<=	ReluAll (	9	downto	5	);
befdly_127	<=	ReluAll (	4	downto	0	);

aftdly_000	<=	ReluAllDly (	639	downto	635	);
aftdly_001	<=	ReluAllDly (	634	downto	630	);
aftdly_002	<=	ReluAllDly (	629	downto	625	);
aftdly_003	<=	ReluAllDly (	624	downto	620	);
aftdly_004	<=	ReluAllDly (	619	downto	615	);
aftdly_005	<=	ReluAllDly (	614	downto	610	);
aftdly_006	<=	ReluAllDly (	609	downto	605	);
aftdly_007	<=	ReluAllDly (	604	downto	600	);
aftdly_008	<=	ReluAllDly (	599	downto	595	);
aftdly_009	<=	ReluAllDly (	594	downto	590	);
aftdly_010	<=	ReluAllDly (	589	downto	585	);
aftdly_011	<=	ReluAllDly (	584	downto	580	);
aftdly_012	<=	ReluAllDly (	579	downto	575	);
aftdly_013	<=	ReluAllDly (	574	downto	570	);
aftdly_014	<=	ReluAllDly (	569	downto	565	);
aftdly_015	<=	ReluAllDly (	564	downto	560	);
aftdly_016	<=	ReluAllDly (	559	downto	555	);
aftdly_017	<=	ReluAllDly (	554	downto	550	);
aftdly_018	<=	ReluAllDly (	549	downto	545	);
aftdly_019	<=	ReluAllDly (	544	downto	540	);
aftdly_020	<=	ReluAllDly (	539	downto	535	);
aftdly_021	<=	ReluAllDly (	534	downto	530	);
aftdly_022	<=	ReluAllDly (	529	downto	525	);
aftdly_023	<=	ReluAllDly (	524	downto	520	);
aftdly_024	<=	ReluAllDly (	519	downto	515	);
aftdly_025	<=	ReluAllDly (	514	downto	510	);
aftdly_026	<=	ReluAllDly (	509	downto	505	);
aftdly_027	<=	ReluAllDly (	504	downto	500	);
aftdly_028	<=	ReluAllDly (	499	downto	495	);
aftdly_029	<=	ReluAllDly (	494	downto	490	);
aftdly_030	<=	ReluAllDly (	489	downto	485	);
aftdly_031	<=	ReluAllDly (	484	downto	480	);
aftdly_032	<=	ReluAllDly (	479	downto	475	);
aftdly_033	<=	ReluAllDly (	474	downto	470	);
aftdly_034	<=	ReluAllDly (	469	downto	465	);
aftdly_035	<=	ReluAllDly (	464	downto	460	);
aftdly_036	<=	ReluAllDly (	459	downto	455	);
aftdly_037	<=	ReluAllDly (	454	downto	450	);
aftdly_038	<=	ReluAllDly (	449	downto	445	);
aftdly_039	<=	ReluAllDly (	444	downto	440	);
aftdly_040	<=	ReluAllDly (	439	downto	435	);
aftdly_041	<=	ReluAllDly (	434	downto	430	);
aftdly_042	<=	ReluAllDly (	429	downto	425	);
aftdly_043	<=	ReluAllDly (	424	downto	420	);
aftdly_044	<=	ReluAllDly (	419	downto	415	);
aftdly_045	<=	ReluAllDly (	414	downto	410	);
aftdly_046	<=	ReluAllDly (	409	downto	405	);
aftdly_047	<=	ReluAllDly (	404	downto	400	);
aftdly_048	<=	ReluAllDly (	399	downto	395	);
aftdly_049	<=	ReluAllDly (	394	downto	390	);
aftdly_050	<=	ReluAllDly (	389	downto	385	);
aftdly_051	<=	ReluAllDly (	384	downto	380	);
aftdly_052	<=	ReluAllDly (	379	downto	375	);
aftdly_053	<=	ReluAllDly (	374	downto	370	);
aftdly_054	<=	ReluAllDly (	369	downto	365	);
aftdly_055	<=	ReluAllDly (	364	downto	360	);
aftdly_056	<=	ReluAllDly (	359	downto	355	);
aftdly_057	<=	ReluAllDly (	354	downto	350	);
aftdly_058	<=	ReluAllDly (	349	downto	345	);
aftdly_059	<=	ReluAllDly (	344	downto	340	);
aftdly_060	<=	ReluAllDly (	339	downto	335	);
aftdly_061	<=	ReluAllDly (	334	downto	330	);
aftdly_062	<=	ReluAllDly (	329	downto	325	);
aftdly_063	<=	ReluAllDly (	324	downto	320	);
aftdly_064	<=	ReluAllDly (	319	downto	315	);
aftdly_065	<=	ReluAllDly (	314	downto	310	);
aftdly_066	<=	ReluAllDly (	309	downto	305	);
aftdly_067	<=	ReluAllDly (	304	downto	300	);
aftdly_068	<=	ReluAllDly (	299	downto	295	);
aftdly_069	<=	ReluAllDly (	294	downto	290	);
aftdly_070	<=	ReluAllDly (	289	downto	285	);
aftdly_071	<=	ReluAllDly (	284	downto	280	);
aftdly_072	<=	ReluAllDly (	279	downto	275	);
aftdly_073	<=	ReluAllDly (	274	downto	270	);
aftdly_074	<=	ReluAllDly (	269	downto	265	);
aftdly_075	<=	ReluAllDly (	264	downto	260	);
aftdly_076	<=	ReluAllDly (	259	downto	255	);
aftdly_077	<=	ReluAllDly (	254	downto	250	);
aftdly_078	<=	ReluAllDly (	249	downto	245	);
aftdly_079	<=	ReluAllDly (	244	downto	240	);
aftdly_080	<=	ReluAllDly (	239	downto	235	);
aftdly_081	<=	ReluAllDly (	234	downto	230	);
aftdly_082	<=	ReluAllDly (	229	downto	225	);
aftdly_083	<=	ReluAllDly (	224	downto	220	);
aftdly_084	<=	ReluAllDly (	219	downto	215	);
aftdly_085	<=	ReluAllDly (	214	downto	210	);
aftdly_086	<=	ReluAllDly (	209	downto	205	);
aftdly_087	<=	ReluAllDly (	204	downto	200	);
aftdly_088	<=	ReluAllDly (	199	downto	195	);
aftdly_089	<=	ReluAllDly (	194	downto	190	);
aftdly_090	<=	ReluAllDly (	189	downto	185	);
aftdly_091	<=	ReluAllDly (	184	downto	180	);
aftdly_092	<=	ReluAllDly (	179	downto	175	);
aftdly_093	<=	ReluAllDly (	174	downto	170	);
aftdly_094	<=	ReluAllDly (	169	downto	165	);
aftdly_095	<=	ReluAllDly (	164	downto	160	);
aftdly_096	<=	ReluAllDly (	159	downto	155	);
aftdly_097	<=	ReluAllDly (	154	downto	150	);
aftdly_098	<=	ReluAllDly (	149	downto	145	);
aftdly_099	<=	ReluAllDly (	144	downto	140	);
aftdly_100	<=	ReluAllDly (	139	downto	135	);
aftdly_101	<=	ReluAllDly (	134	downto	130	);
aftdly_102	<=	ReluAllDly (	129	downto	125	);
aftdly_103	<=	ReluAllDly (	124	downto	120	);
aftdly_104	<=	ReluAllDly (	119	downto	115	);
aftdly_105	<=	ReluAllDly (	114	downto	110	);
aftdly_106	<=	ReluAllDly (	109	downto	105	);
aftdly_107	<=	ReluAllDly (	104	downto	100	);
aftdly_108	<=	ReluAllDly (	99	downto	95	);
aftdly_109	<=	ReluAllDly (	94	downto	90	);
aftdly_110	<=	ReluAllDly (	89	downto	85	);
aftdly_111	<=	ReluAllDly (	84	downto	80	);
aftdly_112	<=	ReluAllDly (	79	downto	75	);
aftdly_113	<=	ReluAllDly (	74	downto	70	);
aftdly_114	<=	ReluAllDly (	69	downto	65	);
aftdly_115	<=	ReluAllDly (	64	downto	60	);
aftdly_116	<=	ReluAllDly (	59	downto	55	);
aftdly_117	<=	ReluAllDly (	54	downto	50	);
aftdly_118	<=	ReluAllDly (	49	downto	45	);
aftdly_119	<=	ReluAllDly (	44	downto	40	);
aftdly_120	<=	ReluAllDly (	39	downto	35	);
aftdly_121	<=	ReluAllDly (	34	downto	30	);
aftdly_122	<=	ReluAllDly (	29	downto	25	);
aftdly_123	<=	ReluAllDly (	24	downto	20	);
aftdly_124	<=	ReluAllDly (	19	downto	15	);
aftdly_125	<=	ReluAllDly (	14	downto	10	);
aftdly_126	<=	ReluAllDly (	9	downto	5	);
aftdly_127	<=	ReluAllDly (	4	downto	0	);

Poolout <= pool_000 & pool_001
		& pool_002 & pool_003
		& pool_004 & pool_005
		& pool_006 & pool_007
		& pool_008 & pool_009

		& pool_010 & pool_011
		& pool_012 & pool_013
		& pool_014 & pool_015
		& pool_016 & pool_017
		& pool_018 & pool_019

		& pool_020 & pool_021
		& pool_022 & pool_023
		& pool_024 & pool_025
		& pool_026 & pool_027
		& pool_028 & pool_029

		& pool_030 & pool_031
		& pool_032 & pool_033
		& pool_034 & pool_035
		& pool_036 & pool_037
		& pool_038 & pool_039

		& pool_040 & pool_041
		& pool_042 & pool_043
		& pool_044 & pool_045
		& pool_046 & pool_047
		& pool_048 & pool_049

		& pool_050 & pool_051
		& pool_052 & pool_053
		& pool_054 & pool_055
		& pool_056 & pool_057
		& pool_058 & pool_059

		& pool_060 & pool_061
		& pool_062 & pool_063
		& pool_064 & pool_065
		& pool_066 & pool_067
		& pool_068 & pool_069

		& pool_070 & pool_071
		& pool_072 & pool_073
		& pool_074 & pool_075
		& pool_076 & pool_077
		& pool_078 & pool_079

		& pool_080 & pool_081
		& pool_082 & pool_083
		& pool_084 & pool_085
		& pool_086 & pool_087
		& pool_088 & pool_089

		& pool_090 & pool_091
		& pool_092 & pool_093
		& pool_094 & pool_095
		& pool_096 & pool_097
		& pool_098 & pool_099

		& pool_100 & pool_101
		& pool_102 & pool_103
		& pool_104 & pool_105
		& pool_106 & pool_107
		& pool_108 & pool_109

		& pool_110 & pool_111
		& pool_112 & pool_113
		& pool_114 & pool_115
		& pool_116 & pool_117
		& pool_118 & pool_119

		& pool_120 & pool_121
		& pool_122 & pool_123
		& pool_124 & pool_125
		& pool_126 & pool_127;

end RTL;

