-- A module of Deep Learning on FPGA
-- by Toshio Iwata at digitalfilter.com 2020/02/23

library ieee ;
use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all ;
use ieee.std_logic_unsigned.all;

-------------------------------------------------------------------------------
-- Entity
-------------------------------------------------------------------------------
entity pooling_max is
port(
  CLK : in std_logic;                            -- clock (100MHz)
  RST_N : in std_logic;                          -- reset (active low)
  Relu0 : in std_logic_vector(4 downto 0); 
  Relu1 : in std_logic_vector(4 downto 0); 
  WR_N : in std_logic; 
  HCO300 : in std_logic_vector(8 downto 0); 
  Poolout : Out std_logic_vector(4 downto 0) );  
end pooling_max;

-------------------------------------------------------------------------------
-- Architecture
-------------------------------------------------------------------------------
architecture RTL of pooling_max is

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal max : std_logic_vector(4 downto 0);
signal shiftreg_00 : std_logic_vector(4 downto 0);
signal shiftreg_01 : std_logic_vector(4 downto 0);
signal shiftreg_10 : std_logic_vector(4 downto 0);
signal shiftreg_11 : std_logic_vector(4 downto 0);

begin
---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		Poolout <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if( WR_N = '0' and HCO300(0) = '1' ) then
			Poolout <= max;
		end if;
	end if;  
end process;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		shiftreg_01 <= (others => '0');
		shiftreg_00 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if( WR_N = '0' ) then
			shiftreg_01 <= shiftreg_00;
			shiftreg_00 <= Relu0;
		end if;
	end if;  
end process;

---------------------------------------------------------
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		shiftreg_11 <= (others => '0');
		shiftreg_10 <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		if( WR_N = '0' ) then
			shiftreg_11 <= shiftreg_10;
			shiftreg_10 <= Relu1;
		end if;
	end if;  
end process;

---------------------------------------------------------
  max <= shiftreg_00 when shiftreg_00 > shiftreg_01 and  shiftreg_00 > shiftreg_10 and  shiftreg_00 > shiftreg_11 else
  		shiftreg_01 when shiftreg_01 > shiftreg_10 and  shiftreg_01 > shiftreg_11 else
  		shiftreg_10 when shiftreg_10 > shiftreg_11 else
		shiftreg_11;

end RTL;

